# Copyright (c) 2024 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .albert import gaudi_albert_forward
from .bart import (
    gaudi_BartAttention_forward,
    gaudi_BartDecoder_forward,
    gaudi_BartDecoderLayer_forward,
    gaudi_BartEncoder_forward,
    gaudi_BartEncoderLayer_forward,
    gaudi_BartForConditionalGeneration_forward,
    gaudi_BartForConditionalGeneration_prepare_inputs_for_generation,
    gaudi_BartLearnedPositionalEmbedding,
    gaudi_BartModel_forward,
)
from .blip import (
    gaudi_BlipForConditionalGeneration_generate,
    gaudi_BlipForQuestionAnswering_generate,
    gaudi_BlipTextAttention_forward,
    gaudi_BlipTextEncoder_forward,
    gaudi_BlipTextLayer_forward,
    gaudi_BlipTextLMHead_forward,
    gaudi_BlipTextLMHead_prepare_inputs_for_generation,
    gaudi_BlipTextModel_forward,
    gaudi_BlipTextSelfAttention_forward,
)
from .bloom import (
    GaudiBloomForCausalLM,
    GaudiBloomMLP,
    gaudi_bloom_attention_forward,
    gaudi_bloom_block_forward,
    gaudi_bloom_convert_to_bloom_cache,
    gaudi_bloom_convert_to_standard_cache,
    gaudi_bloom_model_forward,
)
from .codegen import (
    GaudiCodeGenAttention,
    GaudiCodeGenForCausalLM,
    gaudi_codegen_block_forward,
    gaudi_codegen_model_forward,
)
from .esm import (
    gaudi_esm_for_protein_folding_forward,
    gaudi_esmfolding_trunk_forward,
    gaudi_rot_matmul,
    gaudi_rot_vec_mul,
)
from .falcon import (
    GaudiFalconForCausalLM,
    GaudiFalconModel,
    gaudi_falcon_attention_forward,
    gaudi_falcon_attention_split_heads,
    gaudi_falcon_decoder_layer_forward,
)
from .gpt2 import GaudiGPT2Attention, GaudiGPT2LMHeadModel, gaudi_gpt2_block_forward, gaudi_gpt2_forward
from .gpt_bigcode import (
    GaudiGPTBigCodeForCausalLM,
    gaudi_gpt_bigcode_attention_forward,
    gaudi_gpt_bigcode_block_forward,
    gaudi_gpt_bigcode_model_forward,
)
from .gpt_neox import (
    GaudiGPTNeoXForCausalLM,
    gaudi_gpt_neox_attention_forward,
    gaudi_gpt_neox_layer_forward,
    gaudi_gpt_neox_model_forward,
    gaudi_gpt_neox_rotary_embedding_set_cos_sin_cache,
)
from .gptj import (
    GaudiGPTJAttention,
    GaudiGPTJForCausalLM,
    gaudi_gptj_block_forward,
    gaudi_gptj_model_forward,
)
from .llama import (
    GaudiLlamaAttention,
    GaudiLlamaDecoderLayer,
    GaudiLlamaDynamicNTKScalingRotaryEmbedding,
    GaudiLlamaForCausalLM,
    GaudiLlamaLinearScalingRotaryEmbedding,
    GaudiLlamaMLP,
    GaudiLlamaModel,
    GaudiLlamaRotaryEmbedding,
    gaudi_llama_rmsnorm_forward,
)
from .mistral import (
    GaudiMistralAttention,
    GaudiMistralDecoderLayer,
    GaudiMistralForCausalLM,
    GaudiMistralModel,
    gaudi_mistral_rmsnorm_forward,
)
from .mixtral import (
    GaudiMixtralForCausalLM,
    gaudi_mixtral_attention_forward,
    gaudi_mixtral_block_sparse_moe_forward,
    gaudi_mixtral_decoder_layer_forward,
    gaudi_mixtral_model_forward,
    gaudi_mixtral_rmsnorm_forward,
)
from .modeling_all_models import (
    gaudi_check_and_enable_sdpa,
    gaudi_conv1d_forward,
    gaudi_get_extended_attention_mask,
    gaudi_invert_attention_mask,
)
from .mpt import (
    GaudiMptForCausalLM,
    GaudiMptModel,
    gaudi_mpt_attention_forward,
    gaudi_mpt_block_forward,
)
from .opt import (
    GaudiOPTForCausalLM,
    GaudiOPTLearnedPositionalEmbedding,
    gaudi_opt_attention_forward,
    gaudi_opt_decoder_forward,
    gaudi_opt_decoder_layer_forward,
    gaudi_opt_model_forward,
)
from .phi import (
    GaudiPhiForCausalLM,
    gaudi_phi_attention_forward,
    gaudi_phi_decoder_layer_forward,
    gaudi_phi_model_forward,
)
from .speecht5 import (
    gaudi_generate_speech,
    gaudi_SpeechT5Attention_forward,
    gaudi_SpeechT5Decoder_forward,
    gaudi_SpeechT5DecoderLayer_forward,
    gaudi_SpeechT5SpeechDecoderPrenet_forward,
)
from .swin import gaudi_swin_get_attn_mask
from .t5 import (
    gaudi_t5_layernorm_forward,
    gaudi_T5Attention_forward,
    gaudi_T5Block_forward,
    gaudi_T5ForConditionalGeneration_forward,
    gaudi_T5ForConditionalGeneration_prepare_inputs_for_generation,
    gaudi_T5LayerSelfAttention_forward,
    gaudi_T5Stack_forward,
)
from .vit import gaudi_vit_self_attention_forward
from .wav2vec2 import (
    _gaudi_wav2vec2_compute_mask_indices,
    _gaudi_wav2vec2_mask_hidden_states,
    _gaudi_wav2vec2_sample_negative_indices,
    gaudi_wav2vec2_encoder_forward,
    gaudi_wav2vec2_forward,
    gaudi_wav2vec2_tdnnlayer_forward,
)
